
library("remotes")
library("forecastW2T")
library("forecast")
library("ggplot2")

#Calculate mean average percent error from raw errors and data
#mape <- function(err, data) {100*colMeans(abs(err/data), na.rm = T)}
mape <- function(err, data){
  
  val <- abs(err/data)
  val[!is.finite(val)] <- NA
  val <- 100*colMeans(val, na.rm = T)
  
  return(val)
}


#Set base directory
setwd("/Workspaces/com.casus.env.where2test.forecasting/Nishant")


#Read in a data file. Do this only one at a time
DATA <- read.csv(file="data_training_hospital.csv")

#Check if the data are on cases or tests. If on tests, note that and then change
#the column title to cases, which is what the following code assumes
if(names(DATA)[4]=="hosp"){
  TESTS <- TRUE
  names(DATA)[4] <- "hosp_cases"
} else{
  TESTS <- FALSE
}

#Extract the region codes from the data
CODES <- unique(DATA$region)
L.CODES <- 1:length(CODES)


#Loop over regions, performing a forecast for each region
DATASET <- subset(DATA, DATA$region == CODES[1], select=c("date", "hosp_cases", "cases" ))
#Extract dates and case numbers
DATES <- as.Date(DATASET$date)
CASES <- DATASET$cases


# If we pass external_regressor (in our case 'saxony incidences') as a covariate into the forecast, 
# h will be disregarded and we will get a forecast for length(external_regressor)
# To overcome this issue and forecast four weeks in advance, we must provide forecasted 
# external regressor data (in our case 'saxony incidences')
# for the upcoming four weeks, so that length(new_regressor) == 4 as the covariate of the hospitalization forecast.

# Fit the auto.arima/Arima/Holts/Combination model on the saxony incidences and generate the forecast for saxony incidences of the next four weeks

#cases_fit_autoarima <- auto.arima(DATASET$cases, lambda="auto")
cases_fit_arima <- Arima(DATASET$cases, order=c(1,1,0), lambda="auto")


#Number of simulations. This should be large
NREP <- 10

#Length of the forecast horizon in weeks
HW <- 4

#Generate forecasts via simulation on a weekly level of Saxony incidences with the ARIMA model

#FORECAST_cases_autoarima <- forecast(cases_fit_autoarima, h=4, level=c(95))
#length(FORECAST_cases_autoarima$mean)
SIM_cases <- NULL
for(j in 1:NREP) {SIM_cases <- rbind(SIM_cases, simulate(cases_fit_arima, future=TRUE, nsim=HW))}

# Fit the ARIMA model on the saxony hospitalization incidences 

#hosp_cases_fit <- auto.arima(DATASET$hosp_cases, xreg = DATASET$cases, lambda="auto")
hosp_cases_fit <- Arima(DATASET$hosp_cases, xreg = DATASET$cases, order = c(1,1,0), lambda="auto")

#generate the forecast for saxony hospitalization incidences of the next four weeks using the covariate as the simulation of the incidences

#FORECAST_hosp_cases <- forecast(hosp_cases_fit, xreg=FORECAST_cases_autoarima$mean, level=c(95))
FOR_hosp <- NULL
for(j in 1:NREP) {
  XREG = ts(SIM_cases[j,], start= length(DATASET$cases) + 1, end= length(DATASET$cases) + HW, frequency=1)
  FOR_hosp <- rbind(FOR_hosp, forecast(hosp_cases_fit, xreg=XREG, level = c(95)))
}

#convert the For_hosp from a forecast object to a simple R object
FOR_hosp_ <- NULL
for(j in 1:NREP) {
  FOR_hosp_ <- rbind(FOR_hosp_, FOR_hosp[j,]$mean)
}

#Get the mean and PI limits for the forecasts for the hospitalization data for four horizons
conf_level = 0.95
z = qnorm((1 - conf_level)/2, lower.tail = FALSE)

#calculate the prediction intervals
mean_  <- NULL
lower_ <- NULL
upper_ <- NULL
for(j in 1:HW) {
  mean_  <- rbind(mean_,  mean(FOR_hosp_[,j]))
  sdx    <- sqrt(var(FOR_hosp_[,j])/length(FOR_hosp_[,j]))
  lower_ <- rbind(lower_, mean(FOR_hosp_[,j]) - z * sdx)
  upper_ <- rbind(upper_, mean(FOR_hosp_[,j]) + z * sdx)
}

#Output needs to be a forecast object, so use fitted arima model to get the
#object structure, and overwrite the relevant details with values from combination model
farima <- forecast(Arima(DATASET$hosp_cases, order=c(1,1,0)), h=HW, level=c(95))
hosp_model <- farima

#convert the mean, lower bound and upper bounds into a time series objects so that it can be assigned to the hosp_model forecasting object
mean_  = ts(mean_,  start=length(DATASET$hosp_cases)  + 1, end=length(DATASET$hosp_cases) + HW, frequency=1)
lower_ = ts(lower_, start=length(DATASET$hosp_cases)  + 1, end=length(DATASET$hosp_cases) + HW, frequency=1)
upper_ = ts(upper_, start=length(DATASET$hosp_cases)  + 1, end=length(DATASET$hosp_cases) + HW, frequency=1)


hosp_model$mean   <- mean_
hosp_model$lower  <- lower_
hosp_model$upper  <- upper_
#Give the model and appropriate name
hosp_model$method <- "ARIMA model for hospitalization"


#Title for graphs with model parameter info
#The if logic below automatically changes the fig title based on whether the data
#are on cases or tests
TITLE <- paste0("region= 14000", ", ", "Arima model: (1,1,1)", ", ", "mix= None")

#Plot of data and forecast over time
P1 <- autoplot(hosp_model) + ggtitle(TITLE)
P1
#save the plot to file
#Write plot to file
ggsave("Forecast_Plots.png")

